/**
 * \file crypto_struct_pqcp.h
 *
 * \brief Context structure declarations of the PSA driver that wraps around
 *        mldsa-native.
 */
/*
 *  Copyright The Mbed TLS Contributors
 *  SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 */

#ifndef TF_PSA_CRYPTO_PRIVATE_CRYPTO_STRUCT_PQCP_H
#define TF_PSA_CRYPTO_PRIVATE_CRYPTO_STRUCT_PQCP_H

#include "mbedtls/private_access.h"

#include <psa/crypto_driver_common.h>

#if defined(TF_PSA_CRYPTO_PQCP_MLDSA_ENABLED)

/* No context structures yet. */

#endif /* TF_PSA_CRYPTO_PQCP_MLDSA_ENABLED */

#endif /* TF_PSA_CRYPTO_PRIVATE_CRYPTO_STRUCT_PQCP_H */
